==> trisquel-app-install_0.6.0-3.tar.gz: trisquel-app-install-0.6.0/AppInstall/widgets/ReleaseNotesViewer.py <==
# ReleaseNotesViewer.py
#
#  Copyright (c) 2006 Sebastian Heinlein
#
#  Author: Sebastian Heinlein <sebastian.heinlein@web.de>
#
#  This modul provides an inheritance of the Gtk.TextView that is
#  aware of http URLs and allows to open them in a browser.
#  It is based on the pygtk-demo "hypertext".
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA


import gi
from gi.repository import Gtk, GObject, Gdk
from gi.repository import Pango
import subprocess
import os


class ReleaseNotesViewer(Gtk.TextView):
    def __init__(self, notes=None):
        """Init the ReleaseNotesViewer as an Inheritance of the Gtk.TextView.
           Load the notes into the buffer and make links clickable"""
        # init the parent
        GObject.GObject.__init__(self)
        # global hovering over link state
        self.hovering = False
        self.first = True
        # setup the buffer and signals
        self.set_property("editable", False)
        self.set_cursor_visible(False)
        self.buffer = Gtk.TextBuffer()
        self.set_buffer(self.buffer)
        self.connect("event-after", self.event_after)
        self.connect("motion-notify-event", self.motion_notify_event)
        self.connect("visibility-notify-event", self.visibility_notify_event)
        #self.buffer.connect("changed", self.search_links)
        self.buffer.connect_after("insert-text", self.on_insert_text)
        # search for links in the notes and make them clickable
        if notes != None:
            self.buffer.set_text(notes)

    def tag_link(self, start, end, url):
        """Apply the tag that marks links to the specified buffer selection"""
        tag = self.buffer.create_tag(None, foreground="blue",
                                     underline=Pango.Underline.SINGLE)
        # tag.set_data("url", url) #CHECK
        tag.url = url
        self.buffer.apply_tag(tag, start, end)

    def on_insert_text(self, buffer, iter_end, text, *args):
        """Search for http URLs in newly inserted text  
           and tag them accordingly"""
        itr = buffer.get_iter_at_offset(iter_end.get_offset() - len(text))
        iter_real_end = buffer.get_end_iter()
        for protocol in ["http://", "https://"]:
            while True:
                # search for the next URL in the buffer
                ret = itr.forward_search(protocol,
                                         Gtk.TextSearchFlags.VISIBLE_ONLY,
                                         iter_end)
                # if we reach the end break the loop
                if not ret:
                    break
                # get the position of the protocol prefix
                (match_start, match_end) = ret
                match_tmp = match_end.copy()
                while True:
                    # extend the selection to the complete URL
                    if match_tmp.forward_char():
                        text = match_end.get_text(match_tmp)
                        if text in (" ", ")", "]", "\n", "\t", ">", "!"):
                            break
                    else:
                        break
                    match_end = match_tmp.copy()
                # call the tagging method for the complete URL
                url = match_start.get_text(match_end)
                tags = match_start.get_tags()
                tagged = False
                for tag in tags:
                    url = tag.url
                    if url != "":
                        tagged = True
                        break
                if tagged == False:
                    self.tag_link(match_start, match_end, url)
                # set the starting point for the next search
                itr = match_end

    def event_after(self, text_view, event):
        """callback for mouse click events"""
        # we only react on left mouse clicks
        if event.type != Gdk.EventType.BUTTON_RELEASE:
            return False
        if event.button.button != 1:
            return False

        # try to get a selection
        try:
            (start, end) = self.buffer.get_selection_bounds()
        except ValueError:
            pass
        else:
            if start.get_offset() != end.get_offset():
                return False

        # get the iter at the mouse position
        (x, y) = self.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                              int(event.x), int(event.y))
        itr = self.get_iter_at_location(x, y)[1]

        # call open_url if an URL is assigned to the iter
        tags = itr.get_tags()
        for tag in tags:
            url = tag.url
            if url != None:
                self.open_url(url)
                break

    def open_url(self, url):
        """Open the specified URL in a browser"""
        # Find an appropiate browser
        if os.path.exists('/usr/bin/gnome-open'):
            command = ['gnome-open', url]
        else:
            command = ['x-www-browser', url]

        # Avoid to run the browser as user root
        if os.getuid() == 0 and 'SUDO_USER' in os.environ:
            command = ['sudo', '-u', os.environ['SUDO_USER']] + command

        subprocess.Popen(command)

    def motion_notify_event(self, text_view, event):
        """callback for the mouse movement event, that calls the
           check_hovering method with the mouse postition coordiantes"""
        x, y = text_view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                                 int(event.x), int(event.y))
        self.check_hovering(x, y)
        self.get_window(Gtk.TextWindowType.TEXT).get_pointer()
        return False

    def visibility_notify_event(self, text_view, event):
        """callback if the widgets gets visible (e.g. moves to the foreground)
           that calls the check_hovering method with the mouse position
           coordinates"""
        (w, wx, wy, mod) = text_view.get_window(
            Gtk.TextWindowType.TEXT).get_pointer()
        (bx, by) = text_view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET, wx,
                                                     wy)
        self.check_hovering(bx, by)
        return False

    def check_hovering(self, x, y):
        """Check if the mouse is above a tagged link and if yes show
           a hand cursor"""
        _hovering = False
        # get the iter at the mouse position
        itr = self.get_iter_at_location(x, y)[1]

        # set _hovering if the iter has the tag "url"
        tags = itr.get_tags()
        for tag in tags:
            # CHECK
            if hasattr(tag, 'url'):
                _hovering = True
                break

        # change the global hovering state
        if _hovering != self.hovering or self.first == True:
            self.first = False
            self.hovering = _hovering
            # Set the appropriate cursur icon
            if self.hovering:
                self.get_window(Gtk.TextWindowType.TEXT).\
                    set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
            else:
                self.get_window(Gtk.TextWindowType.TEXT).\
                    set_cursor(Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR))

==> trisquel-app-install_0.6.10.1.tar.gz: trisquel-app-install-0.6.10.1/AppInstall/widgets/ReleaseNotesViewer.py <==
# ReleaseNotesViewer.py
#
#  Copyright (c) 2006 Sebastian Heinlein
#
#  Author: Sebastian Heinlein <sebastian.heinlein@web.de>
#
#  This modul provides an inheritance of the Gtk.TextView that is
#  aware of http URLs and allows to open them in a browser.
#  It is based on the pygtk-demo "hypertext".
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA


import gi
from gi.repository import Gtk, GObject, Gdk
from gi.repository import Pango
import subprocess
import os


class ReleaseNotesViewer(Gtk.TextView):
    def __init__(self, notes=None):
        """Init the ReleaseNotesViewer as an Inheritance of the Gtk.TextView.
           Load the notes into the buffer and make links clickable"""
        # init the parent
        GObject.GObject.__init__(self)
        # global hovering over link state
        self.hovering = False
        self.first = True
        # setup the buffer and signals
        self.set_property("editable", False)
        self.set_cursor_visible(False)
        self.buffer = Gtk.TextBuffer()
        self.set_buffer(self.buffer)
        self.connect("event-after", self.event_after)
        self.connect("motion-notify-event", self.motion_notify_event)
        self.connect("visibility-notify-event", self.visibility_notify_event)
        #self.buffer.connect("changed", self.search_links)
        self.buffer.connect_after("insert-text", self.on_insert_text)
        # search for links in the notes and make them clickable
        if notes != None:
            self.buffer.set_text(notes)

    def tag_link(self, start, end, url):
        """Apply the tag that marks links to the specified buffer selection"""
        tag = self.buffer.create_tag(None, weight=Pango.Weight.BOLD)
        tag.url = url
        self.buffer.apply_tag(tag, start, end)

    def on_insert_text(self, buffer, iter_end, text, *args):
        """Search for http URLs in newly inserted text  
           and tag them accordingly"""
        itr = buffer.get_iter_at_offset(iter_end.get_offset() - len(text))
        iter_real_end = buffer.get_end_iter()
        for protocol in ["http://", "https://"]:
            while True:
                # search for the next URL in the buffer
                ret = itr.forward_search(protocol,
                                         Gtk.TextSearchFlags.VISIBLE_ONLY,
                                         iter_end)
                # if we reach the end break the loop
                if not ret:
                    break
                # get the position of the protocol prefix
                (match_start, match_end) = ret
                match_tmp = match_end.copy()
                while True:
                    # extend the selection to the complete URL
                    if match_tmp.forward_char():
                        text = match_end.get_text(match_tmp)
                        if text in (" ", ")", "]", "\n", "\t", ">", "!"):
                            break
                    else:
                        break
                    match_end = match_tmp.copy()
                # call the tagging method for the complete URL
                url = match_start.get_text(match_end)
                tags = match_start.get_tags()
                tagged = False
                for tag in tags:
                    url = tag.url
                    if url != "":
                        tagged = True
                        break
                if tagged == False:
                    self.tag_link(match_start, match_end, url)
                # set the starting point for the next search
                itr = match_end

    def event_after(self, text_view, event):
        """callback for mouse click events"""
        # we only react on left mouse clicks
        if event.type != Gdk.EventType.BUTTON_RELEASE:
            return False
        if event.button.button != 1:
            return False

        # try to get a selection
        try:
            (start, end) = self.buffer.get_selection_bounds()
        except ValueError:
            pass
        else:
            if start.get_offset() != end.get_offset():
                return False

        # get the iter at the mouse position
        (x, y) = self.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                              int(event.x), int(event.y))
        itr = self.get_iter_at_location(x, y)[1]

        # call open_url if an URL is assigned to the iter
        tags = itr.get_tags()
        for tag in tags:
            if hasattr(tag, 'url'):
                self.open_url(tag.url)
                break

    def open_url(self, url):
        """Open the specified URL in a browser"""
        # Find an appropiate browser
        if os.path.exists('/usr/bin/gnome-open'):
            command = ['gnome-open', url]
        else:
            command = ['x-www-browser', url]

        # Avoid to run the browser as user root
        if os.getuid() == 0 and 'SUDO_USER' in os.environ:
            command = ['sudo', '-u', os.environ['SUDO_USER']] + command

        subprocess.Popen(command)

    def motion_notify_event(self, text_view, event):
        """callback for the mouse movement event, that calls the
           check_hovering method with the mouse postition coordiantes"""
        x, y = text_view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                                 int(event.x), int(event.y))
        self.check_hovering(x, y)
        self.get_window(Gtk.TextWindowType.TEXT).get_pointer()
        return False

    def visibility_notify_event(self, text_view, event):
        """callback if the widgets gets visible (e.g. moves to the foreground)
           that calls the check_hovering method with the mouse position
           coordinates"""
        (w, wx, wy, mod) = text_view.get_window(
            Gtk.TextWindowType.TEXT).get_pointer()
        (bx, by) = text_view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET, wx,
                                                     wy)
        self.check_hovering(bx, by)
        return False

    def check_hovering(self, x, y):
        """Check if the mouse is above a tagged link and if yes show
           a hand cursor"""
        _hovering = False
        # get the iter at the mouse position
        itr = self.get_iter_at_location(x, y)[1]

        # set _hovering if the iter has the tag "url"
        tags = itr.get_tags()
        for tag in tags:
            # CHECK
            if hasattr(tag, 'url'):
                _hovering = True
                break

        # change the global hovering state
        if _hovering != self.hovering or self.first == True:
            self.first = False
            self.hovering = _hovering
            # Set the appropriate cursur icon
            if self.hovering:
                self.get_window(Gtk.TextWindowType.TEXT).\
                    set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
            else:
                self.get_window(Gtk.TextWindowType.TEXT).\
                    set_cursor(Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR))
